from typing import Optional
from mcqq_tool.config import Config as BaseModel


class Config(BaseModel):
    """配置"""
    # 是否开启 @Bot
    mc_qq_to_me: Optional[bool] = True
    # 数据库路径
    mc_qq_dir: Optional[str] = ""



from nonebot import get_driver, ReverseDriver
from nonebot.plugin import PluginMetadata
from nonebot.plugin.manager import PluginLoader
from .config import Config

__plugin_meta__ = PluginMetadata(
    name="青岚Bot",
    description="基于NoneBot的与Minecraft Server互通消息的机器人",
    homepage="https://github.com/17TheWord/qinglan_bot",
    usage="配置完成后在群聊发送消息即可同步至 Minecraft 服务器",
    config=Config,
    type="application",
    supported_adapters={
        "nonebot.adapters.onebot.v11"
    }
)

if isinstance(globals()["__loader__"], PluginLoader):
    driver = get_driver()

    plugin_config: Config = Config.parse_obj(get_driver().config)

    from .router import set_route
    from . import plugins

    # Bot 连接时
    @driver.on_startup
    async def on_start():
        # 启动 WebSocket 服务器
        if isinstance(driver, ReverseDriver):
            await set_route(driver=driver)



from typing import Optional

from aiomcrcon import Client as RconClient

from mcqq_tool.config import CLIENTS, Client
from mcqq_tool.common import plugin_config
from mcqq_tool.utils import send_msg_to_qq, remove_client, rcon_connect

from nonebot import get_bot, logger
from nonebot.exception import WebSocketClosed
from nonebot.drivers import URL, ReverseDriver, WebSocket, WebSocketServerSetup


async def _ws_handler(websocket: WebSocket):
    """WebSocket"""
    try:
        server_name = websocket.request.headers.get("x-self-name").encode('utf-8').decode('unicode_escape')
    except KeyError as e:
        # 服务器名为空
        logger.error(f"[MC_QQ]丨未获取到该服务器的名称，连接断开：{e}")
        await websocket.close(1008, "[MC_QQ]丨未获取到该服务器的名称，连接断开")
        return
    else:

        if CLIENTS.get(server_name):
            # 服务器名已存在
            logger.error(f"[MC_QQ]丨已有相同服务器名的连接，连接断开")
            await websocket.close(1008, "[MC_QQ]丨已有相同服务器名的连接")
            return

        # 准备加入客户端列表
        from .database import DB as db
        rcon_client: Optional[RconClient] = None
        bot_self_id = None
        if server := await db.get_server(server_name=server_name):
            # rcon状态：rcon_msg rcon_cmd rcon_password不等于默认密码
            if (server.rcon_msg or server.rcon_cmd) and server.rcon_password != "change_password":
                bot_self_id = str(server.bot_self_id) if server.bot_self_id else None
                rcon_client = RconClient(server.rcon_ip, server.rcon_port, server.rcon_password)
                await rcon_connect(rcon_client=rcon_client, server_name=server_name)

        CLIENTS[server_name] = Client(server_name=server_name, websocket=websocket, rcon=rcon_client)

        await websocket.accept()

        logger.success(f"[MC_QQ]丨[Server:{server_name}] 已连接至 WebSocket 服务器")

        try:
            while True:
                try:
                    message = await websocket.receive()
                    # 获取指定ID Bot
                    bot = get_bot(bot_self_id)
                except KeyError as e:
                    logger.warning(f"[MC_QQ]丨[Server:{server_name}] 对应 self_id 的 Bot 不存在：{e}")
                except ValueError as e:
                    logger.warning(f"[MC_QQ]丨[Server:{server_name}] 未指定Bot，且当前无其他Bot可用：{e}")
                else:
                    # 以指定ID Bot 发送消息
                    await send_msg_to_qq(bot=bot, message=message)
        except WebSocketClosed as e:
            logger.warning(f"[MC_QQ]丨[Server:{server_name}] 的 WebSocket 出现异常：{e}")
        finally:
            await remove_client(server_name=server_name)


async def set_route(driver: ReverseDriver):
    driver.setup_websocket_server(
        WebSocketServerSetup(
            path=URL(plugin_config.mc_qq_ws_url),
            name="mcqq",
            handle_func=_ws_handler,
        )
    )



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="qinglan-bot",
    version="0.1.5",
    author="17TheWord",
    author_email="17theword@gmail.com",
    description="基于NoneBot的QQ群聊与Minecraft Server消息互通机器人",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/17TheWord/qinglan_bot",
    packages=find_packages(include=['qinglan_bot', 'qinglan_bot.*']),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'mcqq-tool>=0.0.8',
        'nonebot2>=2.0.0',
        'nonebot-adapter-onebot>=2.1.5',
        'nonebot-plugin-guild-patch>=0.2.0',
        'nonebot2[websockets]',
        'websockets>=10.3',
        'tortoise-orm>=0.19.3',
        'aio-mc-rcon>=3.2.0',
        'click>=8.0.4',
    ],
    entry_points={'console_scripts': [
        'ql=qinglan_bot.cli_start:main',
    ]},
)
